# -*- coding: utf-8 -*-
"""
Created on Sun Oct 11 09:33:12 2015

@author: Bruno
"""
try:
    import numpy
except:
    print ("This implementation requires the numpy module.")
    exit(0)
import dataBase
import datetime
import time

from sklearn.metrics import mean_squared_error
from math import sqrt
#handle nan and 0.

def divWith0Handled(a,b):
    """ ignore / 0, div0( [-1, 0, 1], 0 ) -> [0, 0, 0] """
    with numpy.errstate(divide='ignore', invalid='ignore'):
        c = numpy.true_divide( a, b );
        c[ ~ numpy.isfinite( c )] = 0;  # -inf inf NaN
    return c;

from fastFM import mcmc
from fastFM import als
from fastFM import bpr
from fastFM import sgd
from scipy.sparse import csc_matrix

#Constants
TOTAL = "total";
MAX = "max";
TOTAL_MAX = "totalMax";
DATA_TYPE_READ_FLOWITEMS = 0;
DATA_TYPE_CREATED_FLOWITEMS = 1;
DATA_TYPE_READ_CONTAINERITEMS = 2;
DATA_TYPE_CREATED_CONTAINERITEMS = 3;
DATA_TYPE_UPDATED_CONTAINERITEMS = 4;
DATA_TYPE_TOTAL_SESSION_TIME = 5;
DATA_TYPES = [DATA_TYPE_CREATED_FLOWITEMS,DATA_TYPE_READ_FLOWITEMS,DATA_TYPE_READ_CONTAINERITEMS,DATA_TYPE_CREATED_CONTAINERITEMS,DATA_TYPE_UPDATED_CONTAINERITEMS,DATA_TYPE_TOTAL_SESSION_TIME];
INDEX_TYPE_DAY = len(DATA_TYPES) + 1;
INDEX_TYPE_USER = len(DATA_TYPES);
INDEX_TYPE_SPACE = len(DATA_TYPES) + 2;
INDEX_TYPES = [INDEX_TYPE_USER,INDEX_TYPE_DAY,INDEX_TYPE_SPACE];
ALL_DATA_TYPES = INDEX_TYPES + DATA_TYPES;

MAX_RATING = 5;
RATING_TYPE_USER = 0;
RATING_TYPE_SPACE = 1;
RATING_TYPE_RECENT_USER = 2;
RATING_TYPE_RECENT_SPACE = 3;
RATING_TYPES = [RATING_TYPE_USER,RATING_TYPE_SPACE,RATING_TYPE_RECENT_USER,RATING_TYPE_RECENT_SPACE];

#Data
NUMBER_OF_DAYS = 120;
MATRIX_FACTORIZATION_K = 5;

def saveRatingsToServer(ratings, users, spaces):
	finalData = [];
	for userIdx in xrange(len(users)):
		for spaceIdx in xrange(len(spaces)):
			oneFinalData = [users[userIdx], spaces[spaceIdx], ratings[userIdx, spaceIdx]]
			finalData = finalData + [oneFinalData];
	dataBase.sendRatings(finalData);

def saveSuggestedRatingsToServer(ratings, suggestedRatings, users, spaces):
	finalData = [];
	for userIdx in xrange(len(users)):
		for spaceIdx in xrange(len(spaces)):
			if(ratings[userIdx, spaceIdx] == 0):
				oneFinalData = [users[userIdx], spaces[spaceIdx], suggestedRatings[userIdx, spaceIdx]]
				finalData = finalData + [oneFinalData];
	dataBase.sendSuggestedRatings(finalData);

def getDataFromServer():
	data = dataBase.getBigData();
	finalData = [];
	for oneData in data:
		oneFinalData = range(len(ALL_DATA_TYPES));
		for dataTypeIdx in xrange(len(ALL_DATA_TYPES)):
			valor = oneData[dataTypeIdx]
			oneFinalData[ALL_DATA_TYPES[dataTypeIdx]] = valor;
		finalData = finalData + [oneFinalData];
	return finalData;

def saveDataToServer(data):
	finalData = [];
	for oneData in data:
		oneFinalData = range(len(ALL_DATA_TYPES));
		for dataTypeIdx in xrange(len(ALL_DATA_TYPES)):
			valor = oneData[ALL_DATA_TYPES[dataTypeIdx]]
			oneFinalData[dataTypeIdx] = valor;
		finalData = finalData + [oneFinalData];
	dataBase.sendBigData(finalData);

def generateTestDataInServer():
	testData = generateTestData();
	dataBase.dropTables();
	dataBase.createTables();
	saveDataToServer(testData);

def getDateFromNumber(number):
	month = 1;
	if (number > 31):
		number -= 31;
		month +=1;
	if (number > 28):
		number -=28;
		month +=1;
	if (number > 31):
		number -=31;
		month += 1;
	if (number >30):
		number -=30;
		month += 1;
	if (number > 31):
		number -=31;
		month += 1;
	if (number >30):
		number -=30;
		month += 1;
	if (number > 31):
		number -=31;
		month += 1;
	if (number > 31):
		number -=31;
		month += 1;
	if (number >30):
		number -=30;
		month += 1;
	if (number > 31):
		number -=31;
		month += 1;
	if (number >30):
		number -=30;
		month += 1;
	if (number > 31):
		number -=31;
		month += 1;
	while (number >30):
		number -=30;
	return datetime.date(2015,month,number);

def generateTestData():
    numberOfUsers = 100;
    numberOfSpaces = 300;
    numberOfDays = NUMBER_OF_DAYS;
    probabilityOfDayUseUser = 0.9;
    probabilityOfDayUseSpace = 0.7;
    probabilityOfUserUseSpace = 0.7;
    maxData = numpy.zeros(len(DATA_TYPES));
    maxData[DATA_TYPE_CREATED_FLOWITEMS] = 200;
    maxData[DATA_TYPE_READ_FLOWITEMS] = 500;
    maxData[DATA_TYPE_CREATED_CONTAINERITEMS] = 20;
    maxData[DATA_TYPE_READ_CONTAINERITEMS] = 50;
    maxData[DATA_TYPE_UPDATED_CONTAINERITEMS] = 30;
    maxData[DATA_TYPE_TOTAL_SESSION_TIME] = 28800;
    return generateTestDataParametric(numberOfUsers, numberOfSpaces, numberOfDays, probabilityOfDayUseUser, probabilityOfUserUseSpace, probabilityOfDayUseSpace, maxData);

def generateTestDataParametric(numberOfUsers = 10, numberOfSpaces = 20, numberOfDays = 12, probabilityOfDayUseUser = 0.9, probabilityOfUserUseSpace = 0.7, probabilityOfDayUseSpace = 0.7, maxData = [500, 200, 50, 20, 28800]):
    data = [];
    dayUseUser = numpy.random.rand(numberOfDays,numberOfUsers);
    userUseSpace = numpy.random.rand(numberOfUsers,numberOfSpaces);
    for day in range(numberOfDays):
        for user in range(numberOfUsers):
            totalSessionTime = numpy.random.randint(maxData[DATA_TYPE_TOTAL_SESSION_TIME]);
            if (dayUseUser[day,user] < probabilityOfDayUseUser):
                for space in range(numberOfSpaces):
                    if (userUseSpace[user,space] < probabilityOfUserUseSpace and numpy.random.rand() < probabilityOfDayUseSpace):
                        oneData = range(len(ALL_DATA_TYPES));
                        oneData[INDEX_TYPE_USER] ="user " + str(user);
                        oneData[INDEX_TYPE_DAY] = getDateFromNumber(day+1);
                        oneData[INDEX_TYPE_SPACE] = "space " + str(space);
                        oneData[DATA_TYPE_CREATED_FLOWITEMS] = numpy.random.randint(maxData[DATA_TYPE_CREATED_FLOWITEMS]);
                        oneData[DATA_TYPE_READ_FLOWITEMS] = numpy.random.randint(maxData[DATA_TYPE_READ_FLOWITEMS]);
                        oneData[DATA_TYPE_CREATED_CONTAINERITEMS] = numpy.random.randint(maxData[DATA_TYPE_CREATED_CONTAINERITEMS]);
                        oneData[DATA_TYPE_READ_CONTAINERITEMS] = numpy.random.randint(maxData[DATA_TYPE_READ_CONTAINERITEMS]);
                        oneData[DATA_TYPE_UPDATED_CONTAINERITEMS] = numpy.random.randint(maxData[DATA_TYPE_UPDATED_CONTAINERITEMS]);
                        oneData[DATA_TYPE_TOTAL_SESSION_TIME] = totalSessionTime;
                        data.append(oneData);
    return data;
    
def getMaxData(maxData,newData):
    for dataType in DATA_TYPES:
        maxData[dataType] = max([maxData[dataType],newData[dataType]]);
    return maxData;

def cleanBigData(data):
    spacesData = {};
    usersData = {};
    for oneData in data:
        user = oneData[INDEX_TYPE_USER];
        day = oneData[INDEX_TYPE_DAY];
        space = oneData[INDEX_TYPE_SPACE];
        
        dayData = numpy.array(oneData[0:len(DATA_TYPES)],'float');
        #Data of Users
        if (usersData.has_key(user)):
            usersData[user][MAX] = getMaxData(usersData[user][MAX], dayData);
            usersData[user][TOTAL] = usersData[user][TOTAL] + dayData;
        else:
            usersData[user] = {TOTAL:dayData.copy(),MAX:dayData.copy(),TOTAL_MAX:dayData.copy()};
            
        userSpaceData = {};
        if (usersData[user].has_key(space)):
            userSpaceData = usersData[user][space];
            userSpaceData[TOTAL] = userSpaceData[TOTAL] + dayData;
        else:
            userSpaceData[TOTAL] = dayData.copy();
        usersData[user][TOTAL_MAX] = getMaxData(usersData[user][TOTAL_MAX],userSpaceData[TOTAL]);
        userSpaceData[day] = dayData;
        usersData[user][space] = userSpaceData;
        #Data of Space
        spaceData = {};
        if (spacesData.has_key(space)):
            spaceData = spacesData[space];
            spaceData[MAX] = getMaxData(spaceData[MAX],dayData);
            spaceData[TOTAL] = dayData + spaceData[TOTAL];
        else:
            spaceData[MAX] = dayData.copy();
            spaceData[TOTAL] = dayData.copy();
            spaceData[TOTAL_MAX] = dayData.copy();
        spaceData[day] = dayData;
        spaceData[TOTAL_MAX] = getMaxData(spaceData[TOTAL_MAX],userSpaceData[TOTAL]);
        spacesData[space] = spaceData;
            
    return spacesData, usersData;
    
def getRecentData(data,recentLastDays,today):
    day = today;
    result = numpy.zeros(len(DATA_TYPES));
    while day >= today - recentLastDays :
		date = getDateFromNumber(day);
		if (data.has_key(date)):
			result = result + data[date];
		day = day - 1;
    return result;    
    
def getImplicitRatings(spacesData,usersData):
    totalDays = len(spacesData.itervalues().next()) - 2;
    recentLastDays = totalDays * 0.2;
    today = totalDays - 1;

    return getImplicitRatingsDays(spacesData,usersData,recentLastDays,today);

def getImplicitRatingsDays(spacesData,usersData, recentLastDays, today):
    alphasDataTypes = numpy.zeros(len(DATA_TYPES));
    alphasDataTypes[DATA_TYPE_CREATED_FLOWITEMS] = 0.6;
    alphasDataTypes[DATA_TYPE_READ_FLOWITEMS] = 0.3;
    alphasDataTypes[DATA_TYPE_CREATED_CONTAINERITEMS] = 0.8;
    alphasDataTypes[DATA_TYPE_READ_CONTAINERITEMS] = 0.4;
    alphasDataTypes[DATA_TYPE_UPDATED_CONTAINERITEMS] = 0.4;
    alphasDataTypes[DATA_TYPE_TOTAL_SESSION_TIME] = 0.5;
    alphas = numpy.zeros(len(RATING_TYPES));
    alphas[RATING_TYPE_USER] = 0.6;
    alphas[RATING_TYPE_SPACE] = 0.4;
    alphas[RATING_TYPE_RECENT_USER] = 0.3;
    alphas[RATING_TYPE_RECENT_SPACE] = 0.2;
    return getImplicitRatingsParametric(spacesData,usersData,recentLastDays,today,alphas,alphasDataTypes);

def getImplicitRatingsParametric(spacesData,usersData,recentLastDays,today,alphas,alphasDataTypes):
    ratings = numpy.zeros([len(usersData),len(spacesData)]);
    spaces = spacesData.keys();
    users = usersData.keys();
    
    alphaSum = sum(alphas);
    alphaDataTypeSum = sum(alphasDataTypes);
    spaceRatings = [];
    for spaceIdx in xrange(len(spaces)):
        space = spaces[spaceIdx];
        recentSpaceData = getRecentData(spacesData[space],recentLastDays,today);
        userRatings = [];

        for userIdx in xrange(len(users)):
            user = users[userIdx];
            if (usersData[user].has_key(space)):    
                recentUserData = getRecentData(usersData[user][space],recentLastDays,today);
            
                rating = numpy.zeros(len(RATING_TYPES));
                ratingTotal = 0;
                for ratingType in RATING_TYPES:

                    if (ratingType == RATING_TYPE_USER):
                        userRatingTotal = sum(divWith0Handled(usersData[user][space][TOTAL],usersData[user][TOTAL_MAX])*alphasDataTypes);
                        rating[ratingType] = userRatingTotal/alphaDataTypeSum;

                    elif (ratingType == RATING_TYPE_SPACE):
                        spaceRatingTotal = sum(divWith0Handled(usersData[user][space][TOTAL],spacesData[space][TOTAL_MAX])*alphasDataTypes);
                        rating[ratingType] = spaceRatingTotal/alphaDataTypeSum;

                    elif (ratingType == RATING_TYPE_RECENT_USER):
                        userRecentRatingTotal = sum(divWith0Handled(recentUserData,recentSpaceData)*alphasDataTypes);
                        rating[ratingType] = userRecentRatingTotal/alphaDataTypeSum;

                    elif (ratingType == RATING_TYPE_RECENT_SPACE):
                        spaceRecentRatingTotal = sum(divWith0Handled(recentSpaceData,spacesData[space][TOTAL])*alphasDataTypes);
                        rating[ratingType] = spaceRecentRatingTotal/alphaDataTypeSum;

                    ratingTotal = ratingTotal + rating[ratingType] * alphas[ratingType];

                userRatings = userRatings + [rating];
                ratingTotal = ratingTotal / alphaSum;
                ratingTotal = ratingTotal * MAX_RATING;
                
                ratings[userIdx,spaceIdx] = ratingTotal;
            else:
		userRatings = userRatings + [numpy.zeros(len(RATING_TYPES))];


        spaceRatings = spaceRatings + [userRatings];
            
    return ratings,users,spaces,spaceRatings;

def matrixFactorization(R, P, Q, K, steps=500, alpha=0.0002, beta=0.02):
    for step in xrange(steps):
        for i in xrange(len(R)):
            for j in xrange(len(R[i])):
                if R[i][j] > 0:
                    eij = R[i][j] - numpy.dot(P[i,:],Q[:,j]);
                    for k in xrange(K):
                        P[i][k] = P[i][k] + alpha * (2 * eij * Q[k][j] - beta * P[i][k]);
                        Q[k][j] = Q[k][j] + alpha * (2 * eij * P[i][k] - beta * Q[k][j]);
        eR = numpy.dot(P,Q);
        e = 0;
        for i in xrange(len(R)):
            for j in xrange(len(R[i])):
                if R[i][j] > 0:
                    e = e + pow(R[i][j] - numpy.dot(P[i,:],Q[:,j]), 2);
                    for k in xrange(K):
                        e = e + (beta/2) * ( pow(P[i][k],2) + pow(Q[k][j],2) );
        if e < 0.001:
            break
    return P, Q
    
def suggestRatings(ratings):
    N = len(ratings);
    M = len(ratings[0]);
    K = MATRIX_FACTORIZATION_K;

    P = numpy.random.rand(N,K);
    Q = numpy.random.rand(K,M);
    nP, nQ = matrixFactorization(ratings, P, Q, K);
    return numpy.dot(nP,nQ);

def suggestSpacesForUser(userIdentifier,users,spaces,ratings,suggestedRatings):
    suggestions = {};
    userIdx = 0;
    while (userIdx < len(users)):
        if userIdentifier == users[userIdx]:
            for spaceIdx in xrange(len(spaces)):
                if ratings[userIdx,spaceIdx] == 0:
                    suggestions[spaces[spaceIdx]] = suggestedRatings[userIdx,spaceIdx];            
            userIdx = len(users);
        userIdx = userIdx + 1;
    return suggestions;

def initPrinting(printing = 0, debug = 0): 
	data = getDataFromServer();
	spacesData, usersData = cleanBigData(data);
	ratings,users,spaces,debugRatings = getImplicitRatings(spacesData,usersData);
	dataBase.dropRatingsTables();
	dataBase.createRatingsTables();
	saveRatingsToServer(ratings, users, spaces);
	suggestedRatings = suggestRatings(ratings);
	saveSuggestedRatingsToServer(ratings, suggestedRatings, users, spaces);
	if (printing):
		if (debug):
			print "Users:";
			print users;
			print "Spaces:";
			print spaces;
			print "Debug Ratings:";
			print debugRatings
		print "Implicit ratings:";
		print ratings;
		print "\nSuggested ratings:";
		print suggestedRatings;
		print "\nSuggested spaces:";
		for user in users:
		    print "\nSuggested spaces for user: ",user;
		    print suggestSpacesForUser(user,users,spaces,ratings,suggestedRatings);
def init():
	initPrinting(0);


def vectorWithLengthAndData(length,data):
	result = [];
	for i in range(length):
		result = result + [data];
	return result;

def vectorWithLengthFromNumber(length,number):
	result = [];
	for i in range(length):
		result = result + [number + i];
	return result;

def isVector0(vector):
	for x in vector:
		if x != 0:
			return false;
	return true;

def getElementsDistinct0(m,n,array):
	elementsDistinct0 = [];
	for columnIdx in xrange(m):
		for rowIdx in xrange(n):
			if (array[rowIdx,columnIdx] != 0):
				elementsDistinct0 = elementsDistinct0 + [[rowIdx,columnIdx]];
	return elementsDistinct0;

def getCSCMatrixFromArray(array):
	rows = [];
	cols = [];
	data = [];
	rowIdx = 0;
	for row in array:
		colIdx = 0;
		for col in row:
			rows = rows + [rowIdx];
			cols = cols + [colIdx];
			data = data + [col];
			colIdx +=1;
		rowIdx +=1;
	rows = numpy.array(rows);
	cols = numpy.array(cols);
	data = numpy.array(data);
	result = csc_matrix((data, (rows, cols)), shape=(rowIdx, colIdx))
	return result;

def getFastFMEntryFromUsersSpacesRatingsAndDebugRatings(users,spaces,ratings,debugRatings):
	rows = [];
	cols = [];
	data = [];
	rowsTest = [];
	colsTest = [];
	dataTest = [];

	m = len(spaces);
	n = len(users);
	l = len(debugRatings[0][0]);

	featuresIdxs = vectorWithLengthFromNumber(l,m+n);
	rowIdx = 0;
	rowTestIdx = 0;
	y_train = [];
	for spaceIdx in xrange(m):
		space = spaces[spaceIdx];
		for userIdx in xrange(n):
			user = users[userIdx];
			features = debugRatings[spaceIdx][userIdx];
			if (ratings[userIdx,spaceIdx] == 0):
				rowsTest = rowsTest + [rowTestIdx, rowTestIdx];
				colsTest = colsTest + [spaceIdx, m + userIdx];
				dataTest = dataTest + [1,1];
				rowTestIdx += 1;
			else:
				rows = rows + [rowIdx, rowIdx] + vectorWithLengthAndData(l,rowIdx);
				cols = cols + [spaceIdx, m + userIdx] + featuresIdxs;
				data = data + [1,1] + features.tolist();
				y_train = y_train + [ratings[userIdx,spaceIdx]];
				rowIdx += 1;
	rows = numpy.array(rows);
	cols = numpy.array(cols);
	data = numpy.array(data);
	result = csc_matrix((data, (rows, cols)), shape=(rowIdx, m+n+l))
	rows = numpy.array(rowsTest);
	cols = numpy.array(colsTest);
	data = numpy.array(dataTest);
	resultTest = csc_matrix((data, (rows, cols)), shape=(rowTestIdx, m+n+l));
	return result, resultTest, numpy.array(y_train);
	
def getFastFMEntryWithRandomData(users,spaces,ratings,debugRatings,trainSamples =0.8):
	rows = [];
	cols = [];
	data = [];
	rowsTest = [];
	colsTest = [];
	dataTest = [];

	m = len(spaces);
	n = len(users);
	l = len(debugRatings[0][0]);

	elementsDistinct0 = getElementsDistinct0(m,n,ratings);

	numpy.random.shuffle(elementsDistinct0);
	totalOfSamples = len(elementsDistinct0);
	trainSamples = totalOfSamples * trainSamples;

	featuresIdxs = vectorWithLengthFromNumber(l,m+n);
	rowIdx = 0;
	rowTestIdx = 0;

	y_train = [];
	y_test = [];
	for i in range(totalOfSamples):
		userIdx,spaceIdx = elementsDistinct0[i];
		features = debugRatings[spaceIdx][userIdx];
		if (i < trainSamples):
			rows = rows + [rowIdx, rowIdx] + vectorWithLengthAndData(l,rowIdx);
			cols = cols + [spaceIdx, m + userIdx] + featuresIdxs;
			data = data + [1,1] + features.tolist();
			y_train = y_train + [ratings[userIdx,spaceIdx]];
			rowIdx += 1;
		else:
			rowsTest = rowsTest + [rowTestIdx, rowTestIdx] + vectorWithLengthAndData(l,rowTestIdx);
			colsTest = colsTest + [spaceIdx, m + userIdx] + featuresIdxs;
			dataTest = dataTest + [1,1] + features.tolist();
			y_test = y_test + [ratings[userIdx,spaceIdx]];
			rowTestIdx += 1;

	rows = numpy.array(rows);
	cols = numpy.array(cols);
	data = numpy.array(data);
	result = csc_matrix((data, (rows, cols)), shape=(rowIdx, m+n+l))
	rows = numpy.array(rowsTest);
	cols = numpy.array(colsTest);
	data = numpy.array(dataTest);
	resultTest = csc_matrix((data, (rows, cols)), shape=(rowTestIdx, m+n+l));
	return result, resultTest, numpy.array(y_train), numpy.array(y_test);


def testFM(mode = 'sgd', trainSamples = 0.8):
	data = getDataFromServer();
	spacesData, usersData = cleanBigData(data);
	ratings,users,spaces,debugRatings = getImplicitRatings(spacesData,usersData);
	x_train, x_test, y_train, y_test = getFastFMEntryWithRandomData(users,spaces,ratings,debugRatings,trainSamples);
	y_predicted = [];
	if (mode == 'mcmc'):
		fm = mcmc.FMRegression(n_iter=1000,init_stdev=0.1, rank=8, random_state=123);
		y_predicted = fm.fit_predict(x_train, y_train, x_test, n_more_iter=0);
	elif (mode == 'als'):
		fm = als.FMRegression(n_iter=1000,init_stdev=0.1, rank=8, random_state=123);
		fm.fit(x_train, y_train);
		y_predicted = fm.predict(x_test);
	elif (mode == 'sgd'):
		fm = sgd.FMRegression(n_iter=1000,init_stdev=0.1, rank=8, random_state=123);
		fm.fit(x_train, y_train);
		y_predicted = fm.predict(x_test);
	elif (mode == 'rank'):
		import itertools
		pairs = [p for p in itertools.combinations(range(len(y_train)), 2)];
		compares = numpy.zeros((len(pairs), 2), dtype=numpy.float64);

		for i, p in enumerate(pairs):
			if y_train[p[0]] > y_train[p[1]]:
				compares[i, 0] = p[0];
				compares[i, 1] = p[1];
			else:
				compares[i, 0] = p[1];
				compares[i, 1] = p[0];
		fm = bpr.FMRecommender(n_iter=1000,init_stdev=0.1, rank=8, random_state=123);
		fm.fit(x_train, compares);
		y_predicted = fm.predict(x_test);
	
	if (len(y_predicted) == 0):
		print "mode no implemented yet"
		return -1;
	rmse = sqrt(mean_squared_error(y_test, y_predicted));
	return rmse;

def testMF(trainSamples = 0.8):
	data = getDataFromServer();
	spacesData, usersData = cleanBigData(data);
	ratings,users,spaces,debugRatings = getImplicitRatings(spacesData,usersData);
	elements = getElementsDistinct0(len(spaces),len(users),ratings);

	numpy.random.shuffle(elements);
	totalOfSamples = len(elements);
	testSamples = int(totalOfSamples - totalOfSamples * trainSamples);

	y_test = numpy.zeros(testSamples);
	for i in range(testSamples):
		userIdx,spaceIdx = elements[i];
		y_test[i] = ratings[userIdx,spaceIdx];
		ratings[userIdx,spaceIdx] = 0;
	suggestedRatings = suggestRatings(ratings);
	y_predicted = numpy.zeros(testSamples);
	for i in range(testSamples):
		userIdx,spaceIdx = elements[i];
		y_predicted[i] = suggestedRatings[userIdx,spaceIdx];

	rmse = sqrt(mean_squared_error(y_test, y_predicted));
	return rmse;


def rmseMean(mode = 'sgd',iterations = 20):
	if (mode == 'mf'):
		rmse = numpy.zeros(iterations);
		elapsedTime = numpy.zeros(iterations);	
		for i in range(iterations):
			start = time.time();
			rmse[i] = testMF();
			end = time.time();
			elapsedTime[i] = end - start;
		return rmse.mean(),elapsedTime.mean();
	elif (mode == 'sgd' or mode == 'mcmc' or mode == 'als' or mode == 'rank'):
		rmse = numpy.zeros(iterations);
		elapsedTime = numpy.zeros(iterations);	
		for i in range(iterations):
			start = time.time();
			rmse[i] = testFM(mode);
			end = time.time();
			elapsedTime[i] = end - start;
		return elapsedTime.mean(),rmse.mean();
		
def allRMSE():
	print 'rank'
	print rmseMean('rank');
	print 'sgd'
	print rmseMean('sgd');
	print 'mcmc'
	print rmseMean('mcmc');
	print 'als'
	print rmseMean('als');
	print 'mf'
	print rmseMean('mf');


def initWithFastFM(mode = 'sgd'):
	data = getDataFromServer();
	spacesData, usersData = cleanBigData(data);
	ratings,users,spaces,debugRatings = getImplicitRatings(spacesData,usersData);
	x_train, x_test, y_train = getFastFMEntryFromUsersSpacesRatingsAndDebugRatings(users,spaces,ratings,debugRatings);
	y_predicted = [];
	if (mode == 'mcmc'):
		fm = mcmc.FMRegression(n_iter=1000,init_stdev=0.1, rank=8, random_state=123);
		y_predicted = fm.fit_predict(x_train, y_train, x_test, n_more_iter=0);
	elif (mode == 'als'):
		fm = als.FMRegression(n_iter=1000,init_stdev=0.1, rank=8, random_state=123);
		fm.fit(x_train, y_train);
		y_predicted = fm.predict(x_test);
	elif (mode == 'sgd'):
		fm = sgd.FMRegression(n_iter=1000,init_stdev=0.1, rank=8, random_state=123);
		fm.fit(x_train, y_train);
		y_predicted = fm.predict(x_test);
	elif (mode == 'rank'):
		import itertools
		pairs = [p for p in itertools.combinations(range(len(y_train)), 2)];
		compares = numpy.zeros((len(pairs), 2), dtype=numpy.float64);

		for i, p in enumerate(pairs):
			if y_train[p[0]] > y_train[p[1]]:
				compares[i, 0] = p[0];
				compares[i, 1] = p[1];
			else:
				compares[i, 0] = p[1];
				compares[i, 1] = p[0];
		fm = bpr.FMRecommender(n_iter=1000,init_stdev=0.1, rank=8, random_state=123);
		fm.fit(x_train, compares);
		y_predicted = fm.predict(x_test);
	
	if (len(y_predicted) == 0):
		print "mode no implemented yet"
		return -1;
	m = len(spaces);
	n = len(users);
	suggestedRatings = numpy.zeros([n,m]);
	rowIdx = 0;
	for spaceIdx in xrange(m):
		for userIdx in xrange(n):
			if (ratings[userIdx,spaceIdx] == 0):
				suggestedRatings[userIdx,spaceIdx] = y_predicted[rowIdx]*MAX_RATING;
				rowIdx += 1;
	dataBase.dropRatingsTables();
	dataBase.createRatingsTables();
	saveRatingsToServer(ratings, users, spaces);
				
	saveSuggestedRatingsToServer(ratings, suggestedRatings, users, spaces);

