import psycopg2

CONNECTION = "dbname=database user=postgres";
TABLE_USER_METRICS = "user_metrics";
TABLE_IMPLICIT_RATINGS = "implicit_ratings";
TABLE_SUGGESTED_RATINGS = "suggested_ratings";

def importFromCSV(filename,tablename):
	conn = psycopg2.connect(CONNECTION);
	curs = conn.cursor();
	copy_sql = """
		   COPY """+tablename+""" FROM stdin WITH CSV HEADER
		   DELIMITER as ','
		   """
	f = open(filename, 'r');
    	curs.copy_expert(sql=copy_sql, file=f)
	f.close();
	curs.close();
	conn.commit();
	conn.close();	

def exportToCSV(filename,tablename):
	conn = psycopg2.connect(CONNECTION);
	curs = conn.cursor();
	f = open(filename, 'w');
    	curs.copy_to(f,tablename,',')
	f.close();
	curs.close();
	conn.close();	


def executeScript(script):
	conn = psycopg2.connect(CONNECTION);
	curs = conn.cursor();
	curs.execute(script);
	curs.close();
	conn.commit();
	conn.close();


def executeFetchScript(script):
	conn = psycopg2.connect(CONNECTION);
	result = "";
	curs = conn.cursor('curname');
	curs.execute(script);
	result = curs.fetchall();
	curs.close();
	conn.close();
	return result;

def createTables():
	script = "CREATE TABLE "+TABLE_USER_METRICS+" (sf_user_id varchar(100) NOT NULL, sf_date date NOT NULL, space_id varchar(100) NOT NULL, created_flow_items int, read_flow_items int, read_container_items int, created_container_items int, updated_container_items int, total_session_time int, PRIMARY KEY(sf_user_id,sf_date,space_id));"
	
	script += "CREATE TABLE "+TABLE_IMPLICIT_RATINGS+" (sf_user_id varchar(100) NOT NULL, space_id varchar(100) NOT NULL, rating real, PRIMARY KEY(sf_user_id, space_id));"

	script += "CREATE TABLE "+TABLE_SUGGESTED_RATINGS+" (sf_user_id varchar(100) NOT NULL, space_id varchar(100) NOT NULL, rating real, PRIMARY KEY(sf_user_id, space_id));"
	executeScript(script);

def createRatingsTables():
	script = "CREATE TABLE "+TABLE_IMPLICIT_RATINGS+" (sf_user_id varchar(100) NOT NULL, space_id varchar(100) NOT NULL, rating real, PRIMARY KEY(sf_user_id, space_id));"

	script += "CREATE TABLE "+TABLE_SUGGESTED_RATINGS+" (sf_user_id varchar(100) NOT NULL, space_id varchar(100) NOT NULL, rating real, PRIMARY KEY(sf_user_id, space_id));"
	executeScript(script);

def createSuggestedRatingsTable():
	script = "CREATE TABLE "+TABLE_SUGGESTED_RATINGS+" (sf_user_id varchar(100) NOT NULL, space_id varchar(100) NOT NULL, rating real, PRIMARY KEY(sf_user_id, space_id));"
	executeScript(script);

def dropTables():
	script = "DROP TABLE "+TABLE_USER_METRICS+";";
	script += "DROP TABLE "+TABLE_IMPLICIT_RATINGS+";";
	script += "DROP TABLE "+TABLE_SUGGESTED_RATINGS+";";
	executeScript(script);

def dropRatingsTables():
	script = "DROP TABLE "+TABLE_IMPLICIT_RATINGS+";";
	script += "DROP TABLE "+TABLE_SUGGESTED_RATINGS+";";
	executeScript(script);
def dropSuggestedRatingsTable():
	script = "DROP TABLE "+TABLE_SUGGESTED_RATINGS+";";
	executeScript(script);

def stringValuesWithParameters(n):
	values = "(";
	for valorIdx in xrange(n):
		values += "%s";
		if valorIdx != n-1:
			values +=",";
	values += ")";
	return values;

def executeSendData(tableName,data):
	conn = psycopg2.connect(CONNECTION);
	curs = conn.cursor();
	values = stringValuesWithParameters(len(data[0]));
	for oneData in data:
		curs.execute("INSERT INTO "+tableName+" VALUES "+values+";",oneData);
	curs.close();
	conn.commit();
	conn.close();

def sendBigData(data):
	executeSendData(TABLE_USER_METRICS,data);

def sendRatings(data):
	executeSendData(TABLE_IMPLICIT_RATINGS,data);

def sendSuggestedRatings(data):
	executeSendData(TABLE_SUGGESTED_RATINGS,data);

def getDataFromServer(tableName):
	return executeFetchScript("SELECT * FROM "+tableName);

def getBigData():
	return getDataFromServer(TABLE_USER_METRICS);

def getRatings():
	return getDataFromServer(TABLE_IMPLICIT_RATINGS);

def getSuggesttedRatings():
	return getDataFromServer(TABLE_SUGGESTED_RATINGS);

